// gt.c rewritten by Cronos 05/01/2000

#include <stdio.h>
#include <sys/types.h>
#include <winsock.h>

WSADATA wsaData;

int init_winsock(WORD version,BOOL verbose)
{  int err;
	if(verbose) printf("Initialising Winsock\n");
	err = WSAStartup(version, &wsaData);
	if(err)
	{	printf("Could not initialise Winsock\n");
		if(verbose) printf("Error code:%d\n",WSAGetLastError());
	}
	return err;
}

void exit_winsock(void)
{	WSACleanup();
}

struct hostent *s;

void hostent_dump(void)
{	int i;
	printf ("\nResults of host lookup:\n"
			  "Host Name         - %s\n",s->h_name);
	i=0;
	while(s->h_aliases[i]!=NULL)
	{	printf ("Alias List        - %s\n",s->h_aliases[i]);
		i++;
	}
	if(!i) printf("Alias List        - empty\n");
	printf ("Host Address Type - %d\n",s->h_addrtype);
	printf ("Length of Address - %d\n",s->h_length);
	i=0;
	while(((struct in_addr *)(s->h_addr_list[i]))!=NULL)
	{	printf ("List of Addresses from Name Server - %s\n",inet_ntoa(((struct in_addr *)(s->h_addr_list[i]))[0]));
		i++;
	}
}

int hostlookup(char *target,BOOL verbose)
{	unsigned long iadr;
	iadr=inet_addr(target);
	s=NULL;
	if(iadr==INADDR_NONE)
	{	if(verbose) printf("Trying gethostbyname\n");
		s=gethostbyname(target);
	}
	if(s==NULL)
	{  WSASetLastError(0);
		if(verbose) printf("Trying gethostbyaddr\n");
		s=gethostbyaddr((char *)&iadr,4,AF_INET);
	}
	if( s==NULL )
	{	printf("No entry for IP...(Error Code:%d)\n",WSAGetLastError());
		return -1;
	}
	return 0;
}

void helpscreen(void)
{	printf("\nGethost by Cronos - help\n");
	printf("------------------------\n");
	printf("Gethost is a host lookup program which returns all of the results"
			 " given by a host lookup\n");
	printf("Options:\n");
	printf("-h These help options\n");
	printf("-v Verbose mode (more messages)\n");
	printf("Example: gethost -v 208.125.234.22\n");
	printf("Example: gethost modem15.dialup.fserve.net\n");
}

int main(int argc,char *argv[])
{	char *errv,*arv,*opts;
	int argn,tnum,oc;
	BOOL verb,hp;

	if (argc < 2)
	{	errv=argv[0];
		arv=errv;
		while(*errv)
		{	if(*errv=='\\') arv=errv+1;
			errv++;
		}
		printf("Usage: %s -[hv] <target>\n",arv);
		exit(-1);
	}
	verb=FALSE;
	hp=FALSE;
	argn=1;
	tnum=0;
	while(argn<argc)
	{	opts=argv[argn];
		if((opts[0]!='-')&&(!tnum)) tnum=argn;
		else if(opts[0]=='-')
		{	// parse options.
			oc=1;
			while(opts[oc])
			{  switch(opts[oc])
				{	case 'h':
					case 'H':
						hp=TRUE;
						break;
					case 'v':
					case 'V':
						verb=TRUE;
						break;
					default:
						printf("Unknown option:%c\nUse -h for help\n",opts[oc]);
						exit(-1);
				}
				oc++;
			}
		}
		else
		{	printf("Target set as:%s\n",argv[tnum]);
			printf("Found %s in command line as well\n",argv[argn]);
			printf("Use -h for help\n");
			exit(-1);
		}
		argn++;
	}

	if(hp)
	{	helpscreen();
		if(!tnum) exit(0);
	}

	if(!tnum)
	{	printf("No target found in command line\nUse -h for help\n");
		exit(-1);
	}

	if(verb)
	{	printf("Gethost by Cronos\n");
		printf("Target:%s\n",argv[tnum]);
	}

	if(init_winsock(MAKEWORD(1, 1),verb)) exit(-1);

	if(hostlookup(argv[tnum],verb))
	{	exit_winsock();
		exit(-1);
	}
	hostent_dump();

	exit_winsock();
	return 0;
}


